local MAJOR = "StarLibCFG-1.0" 
local MINOR = 1
assert(LibStub, MAJOR.." requires LibStub") 
LibError = LibStub("StarLibError-1.0")
assert(LibError, MAJOR .. " requires StarLibError-1.0")
local LibCFG = LibStub:NewLibrary(MAJOR, MINOR)
if not LibCFG then return end
local evaluator = LibStub("StarLibEvaluator-1.0")
assert(evaluator, MAJOR .. " requires StarLibEvaluator-1.0")

local pool = setmetatable({}, {__mode = "k"})

if not LibCFG.__index then
	LibCFG.__index = LibCFG
end

local TRANSITION_RIGHT = 0
local TRANSITION_LEFT = 1
local TRANSITION_BOTH = 2
local TRANSITION_UP = 3
local TRANSITION_DOWN = 4
local TRANSITION_TENTACLE = 5
local TRANSITION_ALPHABLEND = 6


-- @name StarLibCFG:New
-- @usage StarLibCFG:New(duration, repeating, callback, data)
-- @param config The configuration table
-- @param environment The execution environment
-- @return A new StarLibCFG object
function LibCFG:New(config, environment, errorLevel)
	
	local obj = next(pool)

	if obj then
		pool[obj] = nil
	else
		obj = {}
	end

	setmetatable(obj, self)
			
	obj.environment = environment
	obj.root = config
	obj.error = LibError:New("LibCFG", errorLevel) 
	
	return obj	
	
end

-- @name StarLibCFG:Del
-- @usage StarLibCFG:Del([core]) or object:Del()
-- @param core An optional core object
-- @return Nothing
function LibCFG:Del(core)
	if core and type(core) == "table" then
		pool[core] = true
	else
		pool[self] = true
	end
end

function LibCFG:FetchRaw(section, key, defval)

    local pos = key.find(".");
    if( pos > -1 ) then
        local k = key.substr(0, pos);
        local subSection = CFG_Fetch_Raw(section, k);
        if(not subSection) then
            return defval;
		end
        local root = CFG_Fetch_Raw(subSection, key.sub(pos + 1, strlen(key)), defval);
        return root;
    end

    if(section[key]) then
        return defval;
    end

	local val = section[key] or defval
	
    return val
end

function LibCFG:Fetch(section, key, defval)

	local val = self:FetchRaw(section, key, defval)

    if(not val) then
        return defval
	end;

    if( type(val) == "number") then
        return val;
    elseif ( type(val) == "string" ) then
		return LibEvaluator.ExecuteCode(self.environment, "fetch", val, false, defval)
    end
	
    return defval;
end
	